package main

import (
	"fmt"
	"net/http"

	"database/sql"

	"github.com/gin-gonic/gin"
)

func main() {
	db, err := sql.Open("mysql", "root:@tcp(127.0.0.1:3306)/books")
	defer db.Close()
	if err != nil {
		panic("failed to connect database")
	}

	handler := newHandler(db)

	r := gin.New()

	r.GET("/books", handler.listBooksHandler)
	r.POST("/books", handler.createBookHandler)
	r.DELETE("/books/:id", handler.deleteBookHandler)

	r.Run()
}

type Handler struct {
	db *sql.DB
}

func newHandler(db *sql.DB) *Handler {
	return &Handler{db}
}

type Book struct {
	ID     string `json:"id"`
	Title  string `json:"title"`
	Author string `json:"author"`
}

func (h *Handler) listBooksHandler(c *gin.Context) {
	var books []Book

	rows, err := h.db.Query("SELECT ID,Title,Author FROM Book_Record")
	var id, title, author string

	if err != nil {
		c.JSON(http.StatusInternalServerError, gin.H{
			"error": err,
		})
		return
	}

	for rows.Next() {
		err := rows.Scan(&id, &title, &author)
		if err != nil { /* error handling */
		}
		books = append(books, Book{ID: id, Title: title, Author: author})
	}

	c.JSON(http.StatusOK, &books)
}

func (h *Handler) createBookHandler(c *gin.Context) {
	var book Book

	if err := c.ShouldBindJSON(&book); err != nil {
		c.JSON(http.StatusBadRequest, gin.H{
			"error": err.Error(),
		})
		return
	}

	sql := "INSERT INTO Book_Record(ID,Title,Author) VALUES (?,?,?)"
	insert, err := h.db.Prepare(sql)

	if err != nil {
		panic(err.Error())
	}

	resp, err := insert.Exec(book.ID, book.Title, book.Author)
	insert.Close()

	if err != nil {
		fmt.Println(err)
	}

	lastId, err := resp.LastInsertId()

	if err != nil {
		panic(err)
	}

	fmt.Printf("The last inserted row id: %d\n", lastId)

	c.JSON(http.StatusCreated, &book)
}

func (h *Handler) deleteBookHandler(c *gin.Context) {
	id := c.Param("id")

	_, err := h.db.Exec("delete FROM Book_Record where ID = ?", id)

	if err != nil {
		c.JSON(http.StatusInternalServerError, gin.H{
			"error": err,
		})
		return
	}

	c.Status(http.StatusNoContent)
}
